"""
Provides tlv_play name, title & package information.
"""
TOOL_NAME = 'tlvPlay'
TOOL_TITLE = 'TLV Play'
TOOL_PACKAGE_NAME = 'tlv_play.main'
TOOL_TEST_PACKAGE_NAME = 'tlv_play.test'
MODULE_NAME = 'tlv_play.main.tlvplay'