"""
Provides tlv_play version information.
"""

# This file is auto-generated! Do not edit!
# Use `incremental` to change this file.

from incremental import Version

__version__ = Version("tlv_play", 4, 5, 3)
__all__ = ["__version__"]
